using System;
using UnityEngine;
using UnityEngine.Serialization;

// CUAEditor example
//
// Base editor for a CUA (Custom Unity Asset)
namespace CameraControl
{
    public class CameraControl : MVRScript {
        public JSONStorableBool Cam1Active;
        public JSONStorableBool Cam2Active;
        public JSONStorableBool LedActive;
        public JSONStorableFloat FOV;
        public JSONStorableFloat Cam1Y;
        public JSONStorableFloat Cam2Y;
        public JSONStorableFloat Cam1Z;
        public JSONStorableFloat Cam2Z;
        public JSONStorableFloat Light1Intensity;
        public JSONStorableFloat Light2Intensity;
        public JSONStorableFloat MachineLightIntensity;

        private string _cam1Name = "Camera1";
        private string _cam2Name = "Camera2";
        private string _objectNameSearchString = "Display_Group";
        private string _object2NameSearchString = "WebCam";
        private bool _toggling = false;
        [FormerlySerializedAs("ObjectRoot")] public Transform objectRoot;
        private Transform _cam1;
        private Transform _cam2;

        private Transform _screen1;
        private Transform _screen2;

        private Transform _cameraLed;

        // Security and initialization
        private bool _editorInitialized;
        private int _initLoops;
        private readonly int _maxInitLoops = 3600;
        private Transform _light1;
        private Transform _light2;
        private Transform _machineLight;

        // The usual init function for VAM
        public override void Init()
        {
            try
            {
                // We only authorize the script to be used by a CUA
                if (containingAtom.type != "CustomUnityAsset")
                {
                    LOGError("CUAEditor - Please add this script on a Custom Unity Asset ( script disabled ).");
                    return;
                }

                // A tiny bit of help to give some info to your fellow creator and/or player

                LOGDebug("Init done.");
            }
            catch(Exception e)
            {
                SuperController.LogError("CUAEditor - Exception caught: " + e);
            }
        }

        private void ToggleLed(JSONStorableBool jb) {
            _cameraLed.gameObject.SetActive(jb.val);
        }

        private void DistChange(JSONStorableFloat jf) {
            if (jf.name.Contains("1")) {
                var transform1 = _cam1.transform;
                var v = transform1.localPosition;
                v.y = jf.val;
                transform1.localPosition = v;
            } else {
                var transform1 = _cam2.transform;
                var v = transform1.localPosition;
                v.y = jf.val;
                transform1.localPosition = v;
            }
        }

        private void HeightChange(JSONStorableFloat jf) {
            if (jf.name.Contains("1")) {
                var transform1 = _cam1.transform;
                var v = transform1.localPosition;
                v.z = jf.val;
                transform1.localPosition = v;
            } else {
                var transform1 = _cam2.transform;
                var v = transform1.localPosition;
                v.z = jf.val;
                transform1.localPosition = v;
            }
        }

        private void UpdateFOV(float val) {
            _cam1.GetComponent<Camera>().fieldOfView = val;
            _cam2.GetComponent<Camera>().fieldOfView = val;
        }

        private void ToggleCamera(JSONStorableBool jb) {
            LOGDebug("Firing for " + jb.name + " and " + jb.val);
            if (_toggling) {
                LOGDebug("Untoggling toggle.");
                _toggling = false;
                return;
            }

            if (jb.name.Contains("1")) {
                _cam1.gameObject.SetActive(jb.val);
                _screen1.gameObject.SetActive(jb.val);
                if (jb.val && Cam2Active.val) {
                    _cam2.gameObject.SetActive(false);
                    _screen2.gameObject.SetActive(false);
                    _toggling = true;
                    Cam2Active.val = false;
                }
            } else {
                _cam2.gameObject.SetActive(jb.val);
                _screen2.gameObject.SetActive(jb.val);
                if (jb.val && Cam1Active.val) {
                    _cam1.gameObject.SetActive(false);
                    _screen1.gameObject.SetActive(false);
                    _toggling = true;
                    Cam1Active.val = false;
                }
            }
        }



        // Used for the initialization of the editor.
        // When it is initialized, there's nothing done every frame
        void Update()
        {
            // If the editor is not initialized, we're gonna try to do it
            // I'm putting 3600 tries ( at 60fps it is 1min )
            if( _initLoops < _maxInitLoops && _editorInitialized == false ) {
                _initLoops++;
                InitEditor();
            }

            // Triggering an error if we have reached our limit of tries
            if( _initLoops == _maxInitLoops ) {
                LOGError("CUAEditor : This CUA could not be initialized properly after 3600 tries. Please select a compatible object and reload the script.");
                _initLoops++; // To avoid spamming the log with the error
            }
        }

        // The function initializing everything we need to update the prefab with the editor
        // You can imagine whatever you want here, the only limitations are the things available in VAM's build
        // In this example case, we need the main transform, a transform for the sphere and the cube
        //
        // Initially this function is made to return a bool state to see if it is properly initialized.
        // But since the CUA loader is asynchronous when you reload a scene and I don't have a way to identify when unity's prefab is loaded
        // I have to "brute force" the initialization several times until the script finds the transform I need
        // It implies that your prefab is done properly and has the good name ( this is why the maxInitLoops exists )
        // ReSharper disable Unity.PerformanceAnalysis
        private void InitEditor() {
            // When the editor is initialized, we don't need to do that anymore
            if( _editorInitialized ) return;
            LOGDebug("Editor init.");
            // We need to find our main transform, getObjectRoot is gonna do that for us
            // the search is based on our variable objectNameSearchString
            // you need to create a prefab containing this name in Unity
            // This method works for unparented and parented objects
            // ( Thanks for the reParentObject tip meshed ! )
            objectRoot = GetObjectRoot( containingAtom.reParentObject );
            if( objectRoot == null ) {
                LOGDebug("NULL ROOT.");
                return;
            }

            _cam1 = objectRoot.Find(_cam1Name);
            _cam2 = objectRoot.Find(_cam2Name);
            _screen1 = objectRoot.Find("Screen");
            _screen2 = objectRoot.Find("Screen2");
            _machineLight = GameObject.Find("ExtLight").transform;
            if (_machineLight) LOGDebug("Got machine light.");
            _light1 = GameObject.Find("Light1").transform;
            if (_light1) LOGDebug("Got light 1.");
            _light2 = GameObject.Find("Light2").transform;
            if (_light2) LOGDebug("Got light 2, let's go.");
            var foo = GameObject.Find("Mesh_WebCam_LED");
            _cameraLed = foo.transform;

            // I found my transforms, now the rest
            if( _cam1 && _cam2 && _screen1 && _screen2) {
                LOGDebug("Setting storables...");
                if (_light1 && _light2 && _machineLight) {
                    LOGDebug("This little light o' mine.");
                }
                var cam1State = _cam1.gameObject.activeSelf;
                var cam2State = _cam2.gameObject.activeSelf;

                Cam1Active = new JSONStorableBool("Cam 1 Active", cam1State, ToggleCamera) {
                    storeType = JSONStorableParam.StoreType.Full
                };
                RegisterBool(Cam1Active);

                Cam2Active = new JSONStorableBool("Cam 2 Active", cam2State, ToggleCamera) {
                    storeType = JSONStorableParam.StoreType.Full
                };
                RegisterBool(Cam2Active);

                LedActive = new JSONStorableBool("LED Active", true, ToggleLed) {
                    storeType = JSONStorableParam.StoreType.Full
                };
                RegisterBool(LedActive);

                FOV = new JSONStorableFloat("FOV", 60f, UpdateFOV, 10f, 100f);
                RegisterFloat(FOV);
                _editorInitialized = true;
                LOGDebug("CameraControl: Editor initialized properly.");
                var c1Transform = _cam1.transform.localPosition;
                var c2Transform = _cam2.transform.localPosition;
                Cam1Y = new JSONStorableFloat("Cam 1 Distance", c1Transform.y, DistChange, -3000, 3000);
                RegisterFloat(Cam1Y);

                Cam2Y = new JSONStorableFloat("Cam 2 Distance", c2Transform.y, DistChange, -3000, 3000);
                RegisterFloat(Cam2Y);

                Cam1Z = new JSONStorableFloat("Cam 1 Up/Down", c1Transform.z, HeightChange, -3000, 3000);
                RegisterFloat(Cam1Z);

                Cam2Z = new JSONStorableFloat("Cam 2 Up/Down", c2Transform.z, HeightChange, -3000, 3000);
                RegisterFloat(Cam2Z);

                Light1Intensity = new JSONStorableFloat("Light 1 Intensity", _light1.GetComponent<Light>().intensity, LightChange, 0, 5);
                RegisterFloat(Light1Intensity);
                Light2Intensity = new JSONStorableFloat("Light 2 Intensity", _light2.GetComponent<Light>().intensity, LightChange, 0, 5);
                RegisterFloat(Light2Intensity);

                MachineLightIntensity = new JSONStorableFloat("Machine Light Intensity", _machineLight.GetComponent<Light>().intensity, LightChange, 0, 5);
                RegisterFloat(Cam2Z);

                CreateToggle(Cam1Active);
                CreateSlider(Cam1Y);
                CreateSlider(Cam1Z);
                CreateSlider(Light1Intensity);
                CreateToggle(LedActive);
                CreateSlider(FOV);
                CreateSlider(MachineLightIntensity);
                CreateToggle(Cam2Active, true);
                CreateSlider(Cam2Y, true);
                CreateSlider(Cam2Z, true);
                CreateSlider(Light2Intensity, true);
                var helpText = new JSONStorableString("Help",
                    "<color=#000><size=35><b>Help</b></size></color>\n\n" +
                    "<color=#333>" +
                    "<b>CUAEditor</b> is a tiny plugin to show you how you can create an editor for a CUA ( Custom Unity Asset ).\n\n" +
                    "Change the parameters on the left and you will see the CUA updating in realtime.\n\n" +
                    "<b>IMPORTANT :</b> If you switch objects in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
                    "</color>"
                );
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;
                return;
                // I found my renderers components, let's finish this
            }

            LOGError("CUAEditor: Your CUA is missing mandatory components, or I'm too dumb to find them.");
            _initLoops = _maxInitLoops; // We don't have what we need, we consider the initialization failed
        }

        private void LightChange(JSONStorableFloat jf) {
            Light target;
            if (jf.name.Contains("1")) {
                target = _light1.GetComponent<Light>();
            } else if (jf.name.Contains("2")) {
                target = _light2.GetComponent<Light>();
            } else {
                target = _machineLight.GetComponent<Light>();
            }
            target.intensity = jf.val;
        }


        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        private Transform GetObjectRoot( Transform parent, bool cam = false ) {
            foreach( Transform child in parent ) {
                if( child.name.Contains(cam ? _object2NameSearchString : _objectNameSearchString) ) {
                    return child;
                }

                var childSearch = GetObjectRoot(child);
                if (childSearch != null) return childSearch;
            }

            return null;
        }


        // **************************
        // Local Tools... just because I'm lazy
        // **************************
        private void LOGDebug( string debugText ) {
            SuperController.LogMessage( debugText );
        }

        private void LOGError( string debugText ) {
            SuperController.LogError( debugText );
        }
    }
}
